import math
import timeit
import time


def puissance_naive(a,n) :
    """renvoie a^n par méthode naïve"""
    res = 1
    for i in range(n) :
        res = res * a
    return res 


def exp_binaire(a,n) :
    """renvoie a^n par méthode binaire"""
    res = 1
    m = n
    puiss_a = a 
    while (m != 0) :
        if (m%2 == 1) :
            res = res*puiss_a
        puiss_a = puiss_a * puiss_a
        m = m//2
    return res 


def exp_binaire2(a,n) :
    """renvoie a^n par méthode binaire avec opération sur les bits"""
    res = 1
    m = n
    puiss_a = a 
    while (m != 0 ) :
        if (m & 1) :
            res = res * puiss_a
        m = m>>1
        puiss_a = puiss_a * puiss_a
    return res     


def plus_petit_fact_premiers(n) :
    """renvoie le plus petit facteur premier de n pour n>=2"""
    for i in range(2,1+int(math.sqrt(n))) : 
        if (n%i == 0) :
            return i
    return n


def exp_facteurs(a,n) :
    """renvoie a^n par méthode des facteurs (récursif)"""
    res = 1
    if n == 1 :
        res = a
    else :
        p = plus_petit_fact_premiers(n)
        q = n//p
        res = exp_facteurs(a,p-1) * a
        res = exp_facteurs(res, q) 
    return res 


def exp_rec(a,n) :
    """renvoie a^n par méthode binaire (récursif)"""
    if n == 0 :
        return 1
    res = exp_rec(a,n//2) 
    if n%2 == 0 :
        return res*res
    else :
        return res*res*a
    
#####
##
#
### Les vérifications
#
##

a = 15 #1515
n = 22 #225 #503537

print("a^n par méthode ")
print("*) naïve : ", puissance_naive(a,n))
print("*) binaire : ", exp_binaire(a,n))
print("*) binaire (sur les bits) : ", exp_binaire2(a,n))
print("*) des facteurs : ", exp_facteurs(a,n))
print("*) binaire (récursif) : ", exp_rec(a,n))
print("*) python : ", pow(a,n)) #équivalent à a**n

#
##
#####



#####
##
#
### calculs de temps
#
##


def puissance_python0() :
    t0 = time.clock()
    res = a**n
    t1 = time.clock() 
    return t1 - t0

def puissance_naive0() :
    t0 = time.clock()
    res = puissance_naive(a,n)
    t1 = time.clock() 
    return t1 - t0 

def exp_binaire0() :
    t0 = time.clock()
    res = exp_binaire(a,n)
    t1 = time.clock() 
    return t1 - t0
    
def exp_binaire20() :
    t0 = time.clock()
    res = exp_binaire2(a,n)
    t1 = time.clock() 
    return t1 - t0

def exp_facteurs0() :
    t0 = time.clock()
    res = exp_facteurs(a,n)
    t1 = time.clock() 
    return t1 - t0

def exp_rec0() :
    t0 = time.clock()
    res = exp_rec(a,n)
    t1 = time.clock() 
    return t1 - t0


print("temps d'exécution par méthode ") 
print("*) naïve : ", puissance_naive0())
print("*) binaire : ", exp_binaire0())
print("*) binaire (sur les bits) : ", exp_binaire20()) 
print("*) des facteurs : ", exp_facteurs0())
print("*) binaire (récursif) : ", exp_rec0())
print("*) python : ", puissance_python0())

#
##
#####
